@@ -64,7 +64,7 @@ void set_interface_colorpairs(void)
 			interface_color_pair[index] = COLOR_PAIR(index + 1) | A_BANDAID |
 												combo->attributes;
 		} else {
-			if (index == FUNCTION_TAG)
+			if (index == FUNCTION_TAG || index == SCROLL_BAR)
 				interface_color_pair[index] = A_NORMAL;
 			else if (index == GUIDE_STRIPE)
 				interface_color_pair[index] = A_REVERSE;
@@ -189,8 +189,8 @@ void find_and_prime_applicable_syntax(void)
 	}
 
 #ifdef HAVE_LIBMAGIC
-	/* If we still don't have an answer, try using magic. */
-	if (sntx == NULL && !inhelp) {
+	/* If we still don't have an answer, try using magic (when requested). */
+	if (sntx == NULL && !inhelp && ISSET(USE_MAGIC)) {
 		struct stat fileinfo;
 		magic_t cookie = NULL;
 		const char *magicstring = NULL;
@@ -243,7 +243,7 @@ void find_and_prime_applicable_syntax(void)
 /* Allocate and initialize (for the given line) the cache for multiline info. */
 void set_up_multicache(linestruct *line)
 {
-	line->multidata = (short *)nmalloc(openfile->syntax->nmultis * sizeof(short));
+	line->multidata = nmalloc(openfile->syntax->nmultis * sizeof(short));
 
 	for (short index = 0; index < openfile->syntax->nmultis; index++)
 		line->multidata[index] = -1;
