@@ -48,16 +48,17 @@ static const rcoption rcopts[] = {
 #endif
 	{"casesensitive", CASE_SENSITIVE},
 	{"constantshow", CONSTANT_SHOW},
-	{"emptyline", EMPTY_LINE},
 #ifdef ENABLED_WRAPORJUSTIFY
 	{"fill", 0},
 #endif
 #ifdef ENABLE_HISTORIES
 	{"historylog", HISTORYLOG},
 #endif
-	{"jumpyscrolling", JUMPY_SCROLLING},
 #ifdef ENABLE_LINENUMBERS
 	{"linenumbers", LINE_NUMBERS},
+#endif
+#ifdef HAVE_LIBMAGIC
+	{"magic", USE_MAGIC},
 #endif
 	{"morespace", MORE_SPACE},  /* Deprecated; remove in 2021. */
 #ifdef ENABLE_MOUSE
@@ -93,7 +94,6 @@ static const rcoption rcopts[] = {
 #endif
 	{"suspend", SUSPENDABLE},  /* Deprecated; remove in 2022. */
 	{"suspendable", SUSPENDABLE},
-	{"tabsize", 0},
 	{"tempfile", SAVE_ON_EXIT},  /* Deprecated; remove in 2022. */
 	{"view", VIEW_MODE},
 #ifndef NANO_TINY
@@ -105,8 +105,10 @@ static const rcoption rcopts[] = {
 	{"backupdir", 0},
 	{"bookstyle", BOOKSTYLE},
 	{"cutfromcursor", CUT_FROM_CURSOR},
+	{"emptyline", EMPTY_LINE},
 	{"guidestripe", 0},
 	{"indicator", INDICATOR},
+	{"jumpyscrolling", JUMPY_SCROLLING},
 	{"locking", LOCKING},
 	{"matchbrackets", 0},
 	{"noconvert", NO_CONVERT},
@@ -114,6 +116,8 @@ static const rcoption rcopts[] = {
 	{"smarthome", SMART_HOME},
 	{"smooth", SMOOTH_SCROLL},  /* Deprecated; remove in 2021. */
 	{"softwrap", SOFTWRAP},
+	{"stateflags", STATEFLAGS},
+	{"tabsize", 0},
 	{"tabstospaces", TABS_TO_SPACES},
 	{"trimblanks", TRIM_BLANKS},
 	{"unix", MAKE_IT_UNIX},
@@ -126,6 +130,7 @@ static const rcoption rcopts[] = {
 	{"titlecolor", 0},
 	{"numbercolor", 0},
 	{"stripecolor", 0},
+	{"scrollercolor", 0},
 	{"selectedcolor", 0},
 	{"statuscolor", 0},
 	{"errorcolor", 0},
@@ -612,7 +617,7 @@ bool compile(const char *expression, int rex_flags, regex_t **packed)
 
 	if (outcome != 0) {
 		size_t length = regerror(outcome, compiled, NULL, 0);
-		char *message = charalloc(length);
+		char *message = nmalloc(length);
 
 		regerror(outcome, compiled, message, length);
 		jot_error(N_("Bad regex \"%s\": %s"), expression, message);
@@ -662,7 +667,7 @@ void begin_new_syntax(char *ptr)
 	}
 
 	/* Initialize a new syntax struct. */
-	live_syntax = (syntaxtype *)nmalloc(sizeof(syntaxtype));
+	live_syntax = nmalloc(sizeof(syntaxtype));
 	live_syntax->name = copy_of(nameptr);
 	live_syntax->filename = copy_of(nanorc);
 	live_syntax->lineno = lineno;
@@ -1166,7 +1171,7 @@ void parse_rule(char *ptr, int rex_flags)
 		}
 
 		/* Allocate a rule, fill in the data, and link it into the list. */
-		newcolor = (colortype *)nmalloc(sizeof(colortype));
+		newcolor = nmalloc(sizeof(colortype));
 
 		newcolor->start = start_rgx;
 		newcolor->end = end_rgx;
@@ -1248,7 +1253,7 @@ void grab_and_store(const char *kind, char *ptr, regexlisttype **storage)
 			continue;
 
 		/* Copy the regex into a struct, and hook this in at the end. */
-		newthing = (regexlisttype *)nmalloc(sizeof(regexlisttype));
+		newthing = nmalloc(sizeof(regexlisttype));
 		newthing->full_regex = copy_of(regexstring);
 		newthing->next = NULL;
 
@@ -1551,6 +1556,8 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 			color_combo[LINE_NUMBER] = parse_interface_color(argument);
 		else if (strcmp(option, "stripecolor") == 0)
 			color_combo[GUIDE_STRIPE] = parse_interface_color(argument);
+		else if (strcmp(option, "scrollercolor") == 0)
+			color_combo[SCROLL_BAR] = parse_interface_color(argument);
 		else if (strcmp(option, "selectedcolor") == 0)
 			color_combo[SELECTED_TEXT] = parse_interface_color(argument);
 		else if (strcmp(option, "statuscolor") == 0)
@@ -1578,13 +1585,7 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 		} else
 #endif
 #ifndef NANO_TINY
-		if (strcmp(option, "guidestripe") == 0) {
-			if (!parse_num(argument, &stripe_column) || stripe_column <= 0) {
-				jot_error(N_("Guide column \"%s\" is invalid"), argument);
-				stripe_column = 0;
-			}
-			free(argument);
-		} else if (strcmp(option, "matchbrackets") == 0) {
+		if (strcmp(option, "matchbrackets") == 0) {
 			if (has_blank_char(argument)) {
 				jot_error(N_("Non-blank characters required"));
 				free(argument);
@@ -1621,26 +1622,32 @@ void parse_rcfile(FILE *rcstream, bool just_syntax, bool intros_only)
 			quotestr = argument;
 		else
 #endif
-#ifndef NANO_TINY
-		if (strcmp(option, "backupdir") == 0)
-			backup_dir = argument;
-		else
-		if (strcmp(option, "wordchars") == 0)
-			word_chars = argument;
-		else
-#endif
 #ifdef ENABLE_SPELLER
 		if (strcmp(option, "speller") == 0)
 			alt_speller = argument;
 		else
 #endif
-		if (strcmp(option, "tabsize") == 0) {
+#ifndef NANO_TINY
+		if (strcmp(option, "backupdir") == 0)
+			backup_dir = argument;
+		else if (strcmp(option, "wordchars") == 0)
+			word_chars = argument;
+		else if (strcmp(option, "guidestripe") == 0) {
+			if (!parse_num(argument, &stripe_column) || stripe_column <= 0) {
+				jot_error(N_("Guide column \"%s\" is invalid"), argument);
+				stripe_column = 0;
+			}
+			free(argument);
+		} else if (strcmp(option, "tabsize") == 0) {
 			if (!parse_num(argument, &tabsize) || tabsize <= 0) {
 				jot_error(N_("Requested tab size \"%s\" is invalid"), argument);
 				tabsize = -1;
 			}
 			free(argument);
 		}
+#else
+		;  /* Properly terminate any earlier 'else'. */
+#endif
 	}
 
 	if (intros_only)
